package com.gxzc.zen.common.util

import com.gxzc.zen.common.properties.UploadProperties
import java.io.File
import java.io.FileInputStream
import java.io.IOException
import java.nio.ByteBuffer

/**
 * 文件工具类
 * @author NorthLan
 * @date 2018/5/24
 * @url https://noahlan.com
 */
object FileUtil {

    private var uploadProperties = SpringContextHolder.getBean(UploadProperties::class.java)
        get() {
            if (field == null) {
                field = SpringContextHolder.getBean(UploadProperties::class.java)
            }
            return field
        }

    /**
     * 获取 文件首尾chunk+lastModifiedTime 拼接起来的md5
     */
    fun md5HeadTail(path: String, chunkSize: Int?): String {
        val file = File(path)
        return transientMd5HeadTail(file, chunkSize)
    }

    /**
     * 获取 文件首尾chunk+lastModifiedTime 拼接起来的md5
     */
    fun md5HeadTail(file: File, chunkSize: Int?): String {
        return transientMd5HeadTail(file, chunkSize)
    }

    private fun transientMd5HeadTail(file: File, size: Int?): String {
        var chunkSize = size
        if (chunkSize == null) {
            chunkSize = uploadProperties!!.chunkSize!!.toInt()
        }
        val fc = FileInputStream(file).channel
        val fileSize = fc.size()

        val buffer = ByteBuffer.allocate(chunkSize)
        val lastModified = file.lastModified() / 1000 * 1000 // 由于jdk8的bug导致在linux上读取修改时间丢失精度
        val chunks = Math.ceil(1.0 * fileSize / chunkSize).toInt() // 分块总数

        var currentPos = 0 * chunkSize * 1L

        var retBuffer: ByteBuffer? = null

        var readLength = try {
            fc.read(buffer, currentPos)
        } catch (e: IOException) {
            -1
        }
        if (readLength != -1) {
            val byte = ByteArray(readLength)
            buffer.flip()
            buffer.get(byte)
            buffer.clear()
            // 第一块读完
            currentPos = (chunks - 1) * chunkSize * 1L
            if (currentPos > 0) {
                readLength = try {
                    fc.read(buffer, currentPos)
                } catch (e: IOException) {
                    -1
                }
                if (readLength != -1) {
                    val byte2 = ByteArray(readLength)
                    buffer.flip()
                    buffer.get(byte2)
                    buffer.clear()
                    // 第二块读完
                    retBuffer = ByteBuffer.allocate(byte.size + byte2.size + 8) // 8 为long占用字节数
                    retBuffer.put(byte)
                    retBuffer.put(byte2)
                }
            } else {
                retBuffer = ByteBuffer.allocate(byte.size + 8) // 8 为long占用字节数
                retBuffer.put(byte)
            }
        }

        fc.close() // 关闭文件流

        return if (retBuffer != null) {
            val timeByteArray = ByteBuffer.allocate(8).putLong(lastModified).array()
            timeByteArray.reverse()

            retBuffer.put(timeByteArray)

            MD5Util.encodeMd5(retBuffer.array())
        } else {
            ""
        }

    }
}